/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2008-2011 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at glassfish/bootstrap/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

package com.sun.pkg.bootstrap;

import com.sun.pkg.client.ImagePlanProgressTracker;
import com.sun.pkg.client.Fmri;
import com.sun.pkg.client.Action;

import java.io.Console;
import java.util.logging.Logger;
import java.util.logging.Level;

public class BootstrapProgressTracker extends ImagePlanProgressTracker {

    Fmri pkg = null;
    int totalPackages = 0;
    int totalDownloadFiles = 0;
    long totalDownloadSize = 0;
    Logger logger = null;
    
    // Number of files downloaded so far. We track this separately instead
    // of relying on the file index that is passed to the callbacks because
    // a multi-threaded downloader implementation may download files out
    // of strict order, but the user wants to see nice increasing numbers.
    int nfiles = 0;

    // Get the system console. We use the system console to output verbose
    // progress feedback, like the download file counter. If there is no
    // console then we don't generate that feedback.
    Console console = System.console();

    private BootstrapProgressTracker() {
    }

    public BootstrapProgressTracker(Logger logger) {
        this.logger = logger;
    }

    public void startDownloadPhase(int totalPackages) {
        this.totalPackages = totalPackages;
        logger.log(Level.INFO, "downloading", totalPackages);
    }

    public void startPackageDownload(Fmri pkg, int totalDownloadFiles,
                                     long totalDownloadSize) {
        this.totalDownloadFiles = totalDownloadFiles;
        this.totalDownloadSize = totalDownloadSize;
        this.pkg = pkg;
        this.nfiles = 0;

        logger.log(Level.INFO, "downloadingpackage",
            new Object[] {pkg.getName(),
            totalDownloadFiles,
            totalDownloadSize} );
    }

    public void startFileDownload(int index, long fileSize) {
    }

    public void onFileDownloadProgress(int index, long xferedBytes) {
    }

    public synchronized void endFileDownload(int index, long fileSize) {
        nfiles++;
        if (console != null) {
            console.printf("\015File %d/%d", nfiles, totalDownloadFiles);
        }

    }

    public void endPackageDownload(Fmri pkg, int totalDownloadFiles) {
        if (console != null) {
            console.printf("\n");
        }
    }

    public void endDownloadPhase() {
    }

    public void startActions(int totalActions) {
    }

    public void startRemovalPhase(int totalRemovalActions) {
        if (totalRemovalActions > 0) {
            logger.log(Level.INFO, "executingremovalactions",
                totalRemovalActions);
        }
    }

    public void onRemovalAction(Action a) {
    }

    public void endRemovalPhase() {
    }

    public void startUpdatePhase(int totalUpdateActions) {
        if (totalUpdateActions > 0) {
            logger.log(Level.INFO, "executingupdateactions",
            totalUpdateActions);
        }
    }

    public void onUpdateAction(Action from, Action to) {
        return;
    }

    public void endUpdatePhase() {
    }

    public void startInstallPhase(int totalInstallActions) {
        if (totalInstallActions > 0) {
            logger.log(Level.INFO, "executinginstallactions",
                          totalInstallActions);
        }
    }

    public void onInstallAction(Action a) {
    }

    public void endInstallPhase() {
    }
}
